-- Oracle 10g examples from Chapter 18

DROP TABLE Property;
DROP TABLE Residential;
DROP TABLE Industrial;
DROP TYPE IndustrialType;
DROP TYPE ResidentialType;
DROP TYPE AssessType;
DROP TABLE Agent;
DROP TYPE PropertyType;
DROP TYPE ColorPoint;
DROP TYPE AgentType;
DROP TYPE AddressType;
DROP TYPE Point;
DROP TABLE Agent1;
DROP TYPE AgentType1;


-- Example 18.21

CREATE TYPE Point AS OBJECT
( x FLOAT(15),
  y FLOAT(15),
 MEMBER FUNCTION Distance(P2 Point) RETURN NUMBER,
       -- Computes the distance between 2 points
 MEMBER FUNCTION Equals (P2 Point) RETURN BOOLEAN,
  -- Determines if 2 points are equivalent
 MEMBER PROCEDURE Print )
NOT FINAL 
INSTANTIABLE;
/

-- Example 18.22
CREATE TYPE ColorPoint UNDER Point
(Color INTEGER,
 MEMBER FUNCTION Brighten (Intensity INTEGER) RETURN INTEGER,
  -- Increases color intensity
 MEMBER FUNCTION Equals (CP2 ColorPoint) 
    RETURN BOOLEAN,
  -- Determines if 2 ColorPoints are equivalent
  -- Overriding is not used because the two Equals methods
  -- have different signatures.
 OVERRIDING MEMBER PROCEDURE Print )
NOT FINAL 
INSTANTIABLE;
/

-- Example 18.23
CREATE TYPE AddressType AS OBJECT
( Street  VARCHAR(50),
  City    VARCHAR(30),
  State   CHAR(2),
  Zip     CHAR(9) )
NOT FINAL;
/

CREATE TYPE AgentType AS OBJECT
(AgentNo INTEGER,
 Name    VARCHAR(30),
 Address AddressType,
 Phone   CHAR(13),
 Email   VARCHAR(50) )
 NOT FINAL;
/

CREATE TABLE Agent OF AgentType
( CONSTRAINT AgentPK PRIMARY KEY(AgentNo) )
  OBJECT IDENTIFIER IS SYSTEM GENERATED ;


-- Example 18.24
CREATE TYPE PropertyType AS OBJECT
(PropNo  INTEGER,
 Address AddressType,
 SqFt    INTEGER,
 AgentRef REF AgentType,
 Location Point )
   NOT FINAL
   INSTANTIABLE;
/

CREATE TABLE Property OF PropertyType
 ( CONSTRAINT PropertyPK PRIMARY KEY(PropNo), 
   CONSTRAINT AgentRefFK FOREIGN KEY(AgentRef) REFERENCES Agent )
  OBJECT IDENTIFIER IS SYSTEM GENERATED ;


-- Example 18.25

CREATE TYPE AssessType AS VARRAY(6) OF DECIMAL(9,2);
/

CREATE TYPE ResidentialType UNDER PropertyType 
(BedRooms  INTEGER,
 BathRooms INTEGER,
 Assessments AssessType )
   NOT FINAL
   INSTANTIABLE;
/

CREATE TABLE Residential OF ResidentialType
 (CONSTRAINT ResidentialPK PRIMARY KEY(PropNo), 
  CONSTRAINT AgentRefFK1 FOREIGN KEY(AgentRef) REFERENCES Agent )
  OBJECT IDENTIFIER IS SYSTEM GENERATED ;

CREATE TYPE IndustrialType UNDER PropertyType 
(Zoning  VARCHAR(20),
 AccessDesc VARCHAR(20),
 RailAvailable CHAR(1),
 Parking VARCHAR(10) )
   NOT FINAL
   INSTANTIABLE;
/

CREATE TABLE Industrial OF IndustrialType
(CONSTRAINT IndustrialPK PRIMARY KEY(PropNo), 
 CONSTRAINT AgentRefFK2 FOREIGN KEY(AgentRef) REFERENCES Agent )
 OBJECT IDENTIFIER IS SYSTEM GENERATED ;


-- Example 18.26
INSERT INTO Agent
(AgentNo, Name, Address, Email, Phone)
VALUES (999999, 'Sue Smith', 
        AddressType('123 Any Street', 'Denver', 'CO', '80217'),
         'sue.smith@anyisp.com', '13031234567');

INSERT INTO Agent
(AgentNo, Name, Address, Email, Phone)
VALUES (999998, 'John Smith', 
        AddressType('123 Big Street', 'Boulder', 'CO', '80217'),
         'john.smith@anyisp.com', '13034567123');


-- Example 18.27
INSERT INTO Residential
(PropNo, Address, SqFt, AgentRef, BedRooms, BathRooms, Assessments)
SELECT 999999, AddressType('123 Any Street', 'Denver', 'CO', '80217'), 
       2000, REF(A), 3, 2, AssessType(190000, 200000)
FROM Agent A
WHERE AgentNo = 999999;

-- This INSERT statement maintains set inclusion between the Property
-- and the Residential tables.
INSERT INTO Property
(PropNo, Address, SqFt, AgentRef)
SELECT 999999, AddressType('123 Any Street', 'Denver', 'CO', '80217'), 
       2000, REF(A)
FROM Agent A
WHERE AgentNo = 999999;


-- Example 18.28
UPDATE Residential
  SET AgentRef = 
   ( SELECT REF(A) FROM Agent A WHERE AgentNo = 999998 )
  WHERE PropNo = 999999;

-- This UPDATE statement maintains consistency between the Property
-- and the Residential tables.
UPDATE Property
  SET AgentRef = 
   ( SELECT REF(A) FROM Agent A WHERE AgentNo = 999998 )
  WHERE PropNo = 999999;


-- Example 18.29
SELECT PropNo, P.Address.City, DEREF(AgentRef).Address.City
 FROM Property P
 WHERE DEREF(AgentRef).Name = 'John Smith';


-- Example 18.30
SELECT PropNo, P.Address.City, P.AgentRef.Address.City
 FROM Property P
 WHERE P.AgentRef.Name = 'John Smith';


-- Example 18.31
-- This error causes an error in Oracle 9i and a fault in Oracle 10g because AgentType is not final.
SELECT VALUE(A) FROM Agent A;


-- Example 18.32

CREATE TYPE AgentType1 AS OBJECT
(AgentNo INTEGER,
 Name    VARCHAR(30),
 Phone   CHAR(10),
 Email   VARCHAR(50) );
/
 
CREATE TABLE Agent1 OF AgentType1
( CONSTRAINT Agent1PK PRIMARY KEY(AgentNo) )
  OBJECT IDENTIFIER IS SYSTEM GENERATED ;

INSERT INTO Agent1
(AgentNo, Name, Email, Phone)
VALUES (999998, 'John Smith', 
         'john.smith@anyisp.com', '3037894567');

SELECT VALUE(A) FROM Agent1 A; 

